#!/bin/bash

########################################

PROG_NAME="khomp-log"

REVISION_NUMB="$Rev: 6272 $"
REVISION_DATE="$LastChangedDate: 2011-07-22 12:11:35 -0300 (Fri, 22 Jul 2011) $"

########################################

#DEFAULT_LINE_STRING="-----------------------------------------------------------------------------"
DEFAULT_LINE_STRING=""
CHARSET_TRANSLATION="iconv -f 'UTF-8'"

if ! which 'iconv' &> /dev/null
then
    CHARSET_TRANSLATION="cat"
fi

########################################

FIX_TMP_DIR="/tmp/.khomp-log-run-tmpdir.txt"

msg_prefix ()
{
    echo "[$(date '+%d/%m %H:%M:%S')]"
}

msg_nonl ()
{
    (echo -n "$@" | ${CHARSET_TRANSLATION}) >&2

    local destdir=`cat "${FIX_TMP_DIR}" 2>/dev/null`

    if [ ! "w${destdir}" = "w" -a ! "w${destdir}" = "w/" -a -d "${destdir}" ]
    then
        (echo "$@" | ${CHARSET_TRANSLATION}) >> "${destdir}/MESSAGES.txt"
    fi
}

msg ()
{
    (echo "$@" | ${CHARSET_TRANSLATION}) >&2

    local destdir=`cat "${FIX_TMP_DIR}" 2>/dev/null`

    if [ ! "w${destdir}" = "w" -a ! "w${destdir}" = "w/" -a -d "${destdir}" ]
    then
        (echo "$@" | ${CHARSET_TRANSLATION}) >> "${destdir}/MESSAGES.txt"
    fi
}

# VERIFICA SE EH SUPER-USUARIO
if [ `whoami` != root ]
then
    msg "${PROG_NAME}: ERRO: O script deve ser executado como superusuário."
    exit 1
fi

# VERIFICA SE TEM DEPENDENCIAS
if ! which 'screen' 'find' 'expr' 'asterisk' &> /dev/null
then
    msg "${PROG_NAME}: ERRO: Para ativar os logs da Khomp sao necessarios os seguintes programas instalados: 'screen' 'find' 'expr' 'asterisk'"
    msg "${PROG_NAME}: ERRO: Por favor, verifique se os programas acima foram instalados corretamente ou se a variavel PATH esta setada."
    msg
    msg "${PROG_NAME}: ERRO: Para executar sem a necessidade do programa 'screen', digite '$0 --no-screen'."
    msg "${PROG_NAME}: ERRO: Entretanto, sem o screen, nao sera possível colocar esta sessao em segundo plano."
    exit 1
fi

asterisk_remote ()
{
#   asterisk -r -q -m -x "$@"
    asterisk -r -x "$@"
}

main_tabulate ()
{
    local count="$1"
    local tabs=""

    for ((i=0;i<count;i++))
    do
        tabs="${tabs}  "
    done

    while read LINE
    do
        echo "${tabs}${LINE}"
    done
}

main_find ()
{
    local version="$1"; shift
    find "/var/log/khomp${version}/" -mindepth 1 -maxdepth 1 "$@" | sort -r | head -n1
}

main_search_k3l_api_log ()
{
    local k3lvers="$1"
    local verbose="$2"

    local pidnumb=`pidof kserver-daemon`

    if [ "w${pidnumb}" != "w" ]
    then
        [ "w${verbose}" = "wverbose" ] && \
            msg "$(msg_prefix) Processo KServer encontrado (${pidnumb}), utilizando logs correntes..."

        find "/proc/${pidnumb}/fd" -type l -printf '%l\n' | grep '/var/log/khomp.\../[0-9]\+\.[0-9]\+\.[0-9]\+/[a-zA-Z0-9]\+\.log' | \
            grep -F "khomp${k3lvers}" | sed -e 's#/[a-zA-Z0-9]\+\.log##g' | sort -u
    else
        [ "w${verbose}" = "wverbose" ] && \
            msg "$(msg_prefix) AVISO: Processo KServer NÃO encontrado (${pidnumb}), utilizando últimos logs disponíveis..."

        main_find "${k3lvers}" \( -name '2*-*' -or -name '2*.*.*' \)
    fi
}

main_search_channel_log ()
{
    local k3lvers="$1"
    local verbose="$2"

    local pidnumb=`pidof asterisk`

    if [ "w${pidnumb}" != "w" ]
    then
        [ "w${verbose}" = "wverbose" ] && \
            msg "$(msg_prefix) Processo Asterisk encontrado (${pidnumb}), utilizando logs correntes..."

        find "/proc/${pidnumb}/fd" -type l -printf '%l\n' | grep '/var/log/khomp.\../channel-[0-9]\+_[0-9]\+/[a-zA-Z0-9]\+\.log' | \
            grep -F "khomp${k3lvers}" | sed -e 's#/[a-zA-Z0-9]\+\.log##g' | sort -u
    else
        [ "w${verbose}" = "wverbose" ] && \
            msg "$(msg_prefix) AVISO: Processo Asterisk NÃO encontrado (${pidnumb}), utilizando últimos logs disponíveis..."

        main_find "${k3lvers}" -name 'channel*-*'
    fi
}

main_tar_progress ()
{
    local state=0

    echo -n "  "

    while read
    do
        case "$state" in
          100) echo -ne "\x08|" ;;
          200) echo -ne "\x08/" ;;
          300) echo -ne "\x08-" ;;
          400) echo -ne "\x08\\"; state=0 ;;
        esac

        ((++state))
    done

    echo -e "\x08concluído!"
}

main_log_start ()
{
    local basedir="$1"; shift

    # ACHA A ULTIMA PASTA DE LOGS CRIADA E VERSÃO DA API
    local k3l_summary=`asterisk_remote 'khomp summary' | grep 'K3L API'`

    local k3l_version=`expr "${k3l_summary}" : '.\+K3L API \([0-9.]\+\).\+' | cut -d. -f1-2`

    main_search_k3l_api_log "${k3l_version}" 'verbose' > "${basedir}/path_K3L_API.txt"
    main_search_channel_log "${k3l_version}" 'verbose' > "${basedir}/path_CHANNEL.txt"

    msg # empty line.

    # CRIA PASTAS BASE
    (cd ${basedir} && mkdir -p "khomp/logs" "asterisk/logs") # {asterisk,khomp}/config sao criados posteriormente

    # CRIA LINKS SIMBOLICOS PARA "TAR".
    cat "${basedir}/path_K3L_API.txt" "${basedir}/path_CHANNEL.txt" | \
        while read path
        do
            ln -sf ${path} "${basedir}/khomp/logs/$(basename ${path})"
        done

    msg -n "$(msg_prefix) Ativando logs (versão ${k3l_version}), aguarde..."

    # VARIAVEL MUITO UTILIZADA ABAIXO.
    dest_file="${basedir}/INFO.txt"

    # INFORMACOES DO SISTEMA, VERSOES, FERRAMENTAS

    echo "${PROG_NAME} (${REVISION_NUMB}/${REVISION_DATE})" >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo -n "asterisk -V: " >> "${dest_file}"
    asterisk -V >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}">> "${dest_file}"
    echo "khomp summary:" >> "${dest_file}"

    asterisk_remote "khomp summary" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo -n "khomp revision: " >> "${dest_file}"
    asterisk_remote "khomp revision" | grep '^<K>' >> "${dest_file}"

    msg -n "."


    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo "khomp links errors show:" >> "${dest_file}"

    asterisk_remote "khomp links errors show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo "khomp links show:" >> "${dest_file}"

    asterisk_remote "khomp links show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo "khomp channels show:" >> "${dest_file}"

    asterisk_remote "khomp channels show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

    msg -n "."


    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo "khomp channels statistics show:" >> "${dest_file}"

    asterisk_remote "khomp channels statistics show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo -n "uname -a: " >> "${dest_file}"
    uname -a >> "${dest_file}"

    msg -n "."

    echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
    echo "Ativando logs:" >> "${dest_file}"

    #HABILITA LOGS DE BAIXO NIVEL
    echo "asterisk_remote \"khomp log trace k3l on\"" | main_tabulate 1 >> "${dest_file}"
    asterisk_remote "khomp log trace k3l on" | grep '^<K>' | main_tabulate 2 >> "${dest_file}"

    msg -n "."

    echo >> "${dest_file}"

    #HABILITA OS LOGS DA API/CHANNEL
    echo "asterisk_remote \"khomp log disk debugging\"" | main_tabulate 1 >> "${dest_file}"
    asterisk_remote "khomp log disk debugging" | grep '^<K>' | main_tabulate 2 >> "${dest_file}"

    msg -n "."

    echo >> "${dest_file}"

    # HABILITA TRACE DA SINALIZACAO ISDN
    echo "asterisk_remote \"khomp log trace isdn q931,lapd,system\"" | main_tabulate 1 >> "${dest_file}"
    asterisk_remote "khomp log trace isdn q931,lapd,system" | grep '^<K>' | main_tabulate 2 >> "${dest_file}"

    msg -n "."

    echo >> "${dest_file}"

    # HABILITA TRACE DA SINALIZACAO R2
    echo "asterisk_remote \"khomp log trace r2 all on\"" | main_tabulate 1 >> "${dest_file}"
    asterisk_remote "khomp log trace r2 all on" | grep '^<K>' | main_tabulate 2 >> "${dest_file}"

    msg -n "."

    echo >> "${dest_file}"

    # HABILITA LOGS DO ASTERISK (TODO: CHECAR VERSAO)
    echo "asterisk_remote \"set verbose 99\"" | main_tabulate 1 >> "${dest_file}"
    asterisk_remote "set verbose 99" | main_tabulate 2 >> "${dest_file}"
    asterisk_remote "core set verbose 99" | main_tabulate 2 >> "${dest_file}"

    msg -n "."

    echo >> "${dest_file}"

    echo "asterisk_remote \"set debug 99\"" | main_tabulate 1 >> "${dest_file}"
    asterisk_remote "set debug 99" | main_tabulate 2 >> "${dest_file}"
    asterisk_remote "core set debug 99" | main_tabulate 2 >> "${dest_file}"

    msg -n "."

    echo >> "${dest_file}"

    asterisk_remote "dialplan show" > "${basedir}/DIALPLAN.txt"

    msg -n "."

    local basekhompconfdir="${basedir}/khomp/config"
    local baseastconfdir="${basedir}/asterisk/config"

    # COPIAR CONFIGURACOES
#   mkdir "${basekhompconfdir}"
    cp -rL /etc/khomp/config/ "${basekhompconfdir}"

    if [ -e '/etc/asterisk/asterisk.conf' ]
    then
#       echo '/etc/asterisk' > "${asterisk_file}"
#       mkdir "${baseastconfdir}"
        cp -rL '/etc/asterisk' "${baseastconfdir}"
    else
        local astconfdir=$(dirname `locate extconfig.conf | grep -v 'log' | tail -1`)

        if [ -e "${astconfdir}" ]
        then
#           echo "${astconfdir}" > "${asterisk_file}"
#           mkdir "${baseastconfdir}"
            cp -rL "${astconfdir}" "${baseastconfdir}"
        fi
    fi

    msg "."
    msg "$(msg_prefix) Logs ativados."
}

#################################################################

main_log_stop ()
{
    local basedir="$1"; shift

    msg
    msg -n "$(msg_prefix) Gravando informações adicionais..."

    # VARIAVEL MUITO UTILIZADA ABAIXO.
    dest_file="${basedir}/INFO2.txt"

    # VERIFICA SE O ASTERISK ESTÁ RODANDO
    if ! asterisk_remote "khomp summary" | grep -q ' API '
    then
        msg "."
        msg "$(msg_prefix) Não foi possivel conectar ao asterisk, não há logs para desativar."
    else
        msg -n "."

        echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
        echo "khomp links errors show:" >> "${dest_file}"

        asterisk_remote "khomp links errors show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

        msg -n "."

        echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
        echo "khomp links show:" >> "${dest_file}"

        asterisk_remote "khomp links show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

        msg -n "."

        echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
        echo "khomp channels show:" >> "${dest_file}"

        asterisk_remote "khomp channels show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

        msg -n "."

        echo "${DEFAULT_LINE_STRING}" >> "${dest_file}"
        echo "khomp channels statistics show:" >> "${dest_file}"

        asterisk_remote "khomp channels statistics show" | grep '^<K>' | main_tabulate 1 >> "${dest_file}"

        msg "."

        msg -n "$(msg_prefix) Desativando logs do Asterisk..."

        # DESABILITA LOGS DE BAIXO NIVEL
        asterisk_remote "khomp log trace k3l off" > /dev/null

        msg -n "."

        # DESABILITA LOG DE COMANDOS/EVENTOS DA API E DO CHANNEL DA KHOMP
        asterisk_remote "khomp log disk no debugging" > /dev/null

        msg -n "."

        # DESABILITA TRACE DA SINALIZACAO ISDN
        asterisk_remote "khomp log trace isdn off" > /dev/null

        msg -n "."

        # DESABILITA TRACE DA SINALIZACAO R2
        asterisk_remote "khomp log trace r2 all off" > /dev/null

        msg -n "."

        # DESABILITA LOGS DO ASTERISK
        asterisk_remote "set verbose 0" > /dev/null
        asterisk_remote "core set verbose 0" > /dev/null
        asterisk_remote "set debug 0" > /dev/null
        asterisk_remote "core set debug 0" > /dev/null
    fi

    msg "."

    # APAGA ARQUIVOS TEMPORARIOS
    rm -f "${basedir}/path_K3L_API.txt"
    rm -f "${basedir}/path_CHANNEL.txt"

    local baseastdir="${basedir}/asterisk/logs"

    # FILTRA OS LOGS DO ASTERISK
    if [ -r '/var/log/asterisk/full' -o -r '/var/log/asterisk/messages' ]
    then
#       mkdir "${baseastdir}"

        if [ -r '/var/log/asterisk/full' ]
        then
            ln -s '/var/log/asterisk/full' "${baseastdir}/full.txt"
        fi

        if [ -r '/var/log/asterisk/messages' ]
        then
            ln -s '/var/log/asterisk/messages' "${baseastdir}/messages.txt"
        fi
    else
        msg "$(msg_prefix) AVISO: Nenhum log do Asterisk encontrado, ignorando..."
    fi

    msg_nonl "$(msg_prefix) Copiando os logs, aguarde..."

    local tmp_date_time=`date '+%Y%m%d-%H%M%S'`
    local tmp_base_name="khomp-${tmp_date_time}"
    local tmp_path_name="/tmp/${tmp_base_name}"
    local tmp_file_name="${tmp_base_name}.tar.gz"
    local tmp_full_name="${tmp_path_name}.tar.gz"

    # opções tar: dereferencia symlinks, compacta, gzipa, passa nome de arquivo.
    ( cd /tmp && mv $(basename "${basedir}") ${tmp_base_name} && \
        tar --checkpoint -hczf "${tmp_file_name}" "${tmp_base_name}" 2>&1 | \
            main_tar_progress 1>&2 )

    # SAVE FILE NAME...
    echo "${tmp_full_name}" > "${tmp_path_name}/filename.txt"

    # SAVE NEW TEMPORARY DIR
    echo "${tmp_path_name}" > "${FIX_TMP_DIR}"

    msg # empty line.

    msg "$(msg_prefix) Os logs foram desativados e copiados."
}

main_log_loop ()
{
    local basedir="$1"; shift
    local withscr="$1"; shift

    dest_file="${basedir}/SYSDATA.txt"

    msg
    msg "$(msg_prefix) Sistema de logs inicializado e operando."

    if [ "${withscr}" = "yes" ]
    then
        msg
        msg "$(msg_prefix) Pressione Ctrl-A D para colocar este console em segundo plano."
        msg "$(msg_prefix) (para retornar, utilize o comando $0)"
    fi

    msg
    msg "$(msg_prefix) Pressione 'Q' ou 'q' para finalizar os logs e empacotá-los para envio..."

    echo "Initial time : `date` \n" >> "${dest_file}"
    echo >> "${dest_file}"

    echo "${DEFAULT_LINE_STRING} /etc/issue" >> "${dest_file}"
    cat '/etc/issue' >> "${dest_file}"
    echo >> "${dest_file}"

    echo "\n ${DEFAULT_LINE_STRING} /proc/cpuinfo" >> "${dest_file}"
    cat '/proc/cpuinfo' >> "${dest_file}"
    echo >> "${dest_file}"

    echo "${DEFAULT_LINE_STRING} /proc/meminfo" >> "${dest_file}"
    cat '/proc/meminfo' >> "${dest_file}"
    echo >> "${dest_file}"

    echo "${DEFAULT_LINE_STRING} /proc/interrupts" >> "${dest_file}"
    cat '/proc/interrupts' >> "${dest_file}"
    echo >> "${dest_file}"

    echo "${DEFAULT_LINE_STRING} /proc/modules" >> "${dest_file}"
    cat '/proc/modules' >> "${dest_file}"
    echo >> "${dest_file}"

    echo "${DEFAULT_LINE_STRING} initial ps" >> "${dest_file}"
    ps -eo 'pid,tid,class,rtprio,ni,pri,psr,pcpu,stat,args' >> "${dest_file}"
    echo >> "${dest_file}"

    local is_running="true"

    while true
    do
        for i in 0 1 2 3
        do
            if read -s -n1 -t15 CHAR
            then
                if [ "w${CHAR}" = "wq" -o "w${CHAR}" = "wQ" ]
                then
                    return 0
                fi
            fi

            local k3l_summary=`asterisk_remote 'khomp summary' 2>/dev/null | grep 'K3L API'`

            if [ "w${k3l_summary}" = "w" ]
            then
                if [ "${is_running}" = "true" ]
                then
                    msg ""
                    msg "$(msg_prefix) Não foi possível conectar ao Asterisk, aguardando..."
                fi

                continue
            else
                if [ "${is_running}" = "false" ]
                then
                    is_running="true"
                fi
            fi

            local k3l_version=`expr "${k3l_summary}" : '.\+K3L API \([0-9.]\+\).\+' | cut -d. -f1-2`

            main_search_k3l_api_log "${k3l_version}" > "${basedir}/newpath_K3L_API.txt"

            if grep -qvxF -f "${basedir}/path_K3L_API.txt" "${basedir}/newpath_K3L_API.txt" 2>/dev/null
            then
                cat "${basedir}/newpath_K3L_API.txt" | \
                    while read path
                    do
                        ln -sf ${path} "${basedir}/khomp/logs/$(basename ${path})"
                    done

                msg "$(msg_prefix) Novo diretório de logs da Khomp, marcando para empacotamento..."

                # CRIA LINKS SIMBOLICOS PARA "TAR" E SALVA PARA COMPARACAO FUTURA
                cat "${basedir}/newpath_K3L_API.txt" >> "${basedir}/path_K3L_API.txt"
            fi

            rm -f "${basedir}/newpath_K3L_API.txt"

            main_search_channel_log "${k3l_version}" > "${basedir}/newpath_CHANNEL.txt"

            if grep -qvxF -f "${basedir}/path_CHANNEL.txt" "${basedir}/newpath_CHANNEL.txt" 2>/dev/null
            then
                cat "${basedir}/newpath_CHANNEL.txt" | \
                    while read path
                    do
                        ln -sf ${path} "${basedir}/khomp/logs/$(basename ${path})"
                    done

                msg "$(msg_prefix) Novo diretório de logs do channel, marcando para empacotamento..."

                # CRIA LINKS SIMBOLICOS PARA "TAR" E SALVA PARA COMPARACAO FUTURA
                cat "${basedir}/newpath_CHANNEL.txt" >> "${basedir}/path_CHANNEL.txt"
            fi

            rm -f "${basedir}/newpath_CHANNEL.txt"
        done

        time_string=`date +'%D--%H:%M:%S'`

        echo "${DEFAULT_LINE_STRING} loadavg (${time_string})" >> "${dest_file}"
        cat '/proc/loadavg' >> "${dest_file}"

        echo >> "${dest_file}"

        echo "${DEFAULT_LINE_STRING} ps (${time_string})" >> "${dest_file}"
        ps -eo 'pid,psr,pcpu,stat,rss,size,vsize,pending,thcount,wchan,comm' >> "${dest_file}"

        echo >> "${dest_file}"
    done
}

main_run ()
{
    local tmpdir="$1"; shift
    local scrstt="$1"; shift

    main_log_start "${tmpdir}" "${scrstt}"
    main_log_loop  "${tmpdir}" "${scrstt}"
    main_log_stop  "${tmpdir}" "${scrstt}"
}

magical_string="screened"

if [ "$1" = "${magical_string}" ]
then
    # INICIA LOGS!
    main_run "$2" "yes"
    sleep 1
else
    if ! asterisk_remote "khomp log console no all" > /dev/null
    then
        msg "$0: ERRO: Necessário que o Asterisk esteja em execução para ativar os logs."
        exit 1
    fi

    tmpdir=`cat ${FIX_TMP_DIR} 2> /dev/null`

    if [ "w${tmpdir}" = "w" -o "w${tmpdir}" = "w/" -o ! -d "${tmpdir}" ]
    then
        tmpdir=`mktemp -d -p "/tmp" '.khomp-log.XXXXXX'`

        if [ "w${tmpdir}" = "w" -o "w${tmpdir}" = "w/" ]
        then
            msg "${PROG_NAME}: ERRO: Impossível criar diretório temporário em '/tmp'."
            exit 1
        fi

        if [ ! -d "${tmpdir}" ]
        then
            msg "${PROG_NAME}: ERRO: Impossível criar diretório temporário em '/tmp', verifique o espaço em disco!"
            exit 1
        fi

        echo "${tmpdir}" > "${FIX_TMP_DIR}"
    fi

    if [ "$1" = "--no-screen" ]
    then
        # SEM SCREEN :-(
        main_run "${tmpdir}" "no"
        sleep 1
    else
        # INICIA SCREEN!
        export PATH="${PATH}:."
        screen -D -R -S "khomplogrun" -t "khomp-log" -s "khomp-log" "$0" "${magical_string}" "${tmpdir}"
    fi

    newtmpdir=`cat "${FIX_TMP_DIR}" 2>/dev/null`

    if [ "w${newtmpdir}" = "w" -o "w${newtmpdir}" = "w/" -o ! -d "${newtmpdir}" ]
    then
        msg "${PROG_NAME}: ERRO: Impossível encontrar pasta necessária para armazenamento dos logs!"
        exit 1
    else
        if [ ! -f "${newtmpdir}/filename.txt" ]
        then
            # still not finished...
            exit 0
        fi

        # GET FILE NAME...
        tmp_file_name=`cat "${newtmpdir}/filename.txt" 2>/dev/null`

        if [ ! -f "${tmp_file_name}" ]
        then
            msg "${PROG_NAME}: ERRO: Impossível encontrar arquivo necessário para armazenamento dos logs."
            exit 0
        fi

        msg
        msg "Os logs foram salvos com sucesso!"
        msg
        msg "Favor enviar o arquivo '${tmp_file_name}' para 'suporte@khomp.com.br'"
        msg

        [ -d "${tmpdir}" ]    && rm -Rf "${tmpdir}"
        [ -d "${newtmpdir}" ] && rm -Rf "${newtmpdir}"

        rm -f "${FIX_TMP_DIR}"
    fi
fi
